@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005 Rocky Bernstein <rocky@panix.com>
+ * Copyright (C) 2005, 2006 Rocky Bernstein <rocky@panix.com>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -13,8 +13,8 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
  */
 /* Simple program to show using libcdio's version of the CD-DA paranoia. 
    library. 
@@ -24,7 +24,7 @@
 # include "config.h"
 #endif
 
-#include <cdio/cdda.h>
+#include <cdio/paranoia.h>
 #include <cdio/cd_types.h>
 #include <stdio.h>
 
@@ -53,6 +53,11 @@ main(int argc, const char *argv[])
   /* Don't need a list of CD's with CD-DA's any more. */
   cdio_free_device_list(ppsz_cd_drives);
 
+  if ( !d ) {
+    printf("Unable to identify audio CD disc.\n");
+    exit(1);
+  }
+
   /* We'll set for verbose paranoia messages. */
   cdda_verbose_set(d, CDDA_MESSAGE_PRINTIT, CDDA_MESSAGE_PRINTIT);
 
@@ -112,4 +117,3 @@ main(int argc, const char *argv[])
 
   exit(0);
 }
-
