@@ -1,8 +1,8 @@
 /*
-    $Id: iso9660.h,v 1.90 2006/03/18 02:35:07 rocky Exp $
+    $Id: iso9660.h,v 1.95 2006/10/27 10:38:41 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2003, 2004, 2005, 2006 Rocky Bernstein <rocky@gnu.org>
 
     See also iso9660.h by Eric Youngdale (1993).
 
@@ -300,9 +300,12 @@ struct iso9660_pvd_s {
                                                     table */
   iso732_t         opt_type_m_path_table;        /**< first sector of optional
                                                     M Path table */
-  iso9660_dir_t    root_directory_record;        /**< See section 9.1 of
+  iso9660_dir_t    root_directory_record;        /**< See 8.4.18 and 
+                                                    section 9.1 of
                                                     ISO 9660 spec. */
-  char             root_directory_filename;      /**< Is '\\0' */
+  char             root_directory_filename;      /**< Is '\\0' or root
+                                                  directory. Also pads previous
+                                                  field to 34 bytes */
   dchar_t          volume_set_id[ISO_MAX_VOLUMESET_ID]; /**< Volume Set of 
                                                            which the volume is
                                                            a member. See 
@@ -389,8 +392,10 @@ struct iso9660_svd_s {
                                                   */
   iso711_t         version;                      /**< value 1 */
   char             flags;			 /**< Section 8.5.3 */
-  achar_t          system_id[ISO_MAX_SYSTEM_ID]; /**< each char is an achar */
-  dchar_t          volume_id[ISO_MAX_VOLUME_ID]; /**< each char is a dchar */
+  achar_t          system_id[ISO_MAX_SYSTEM_ID]; /**< Section 8.5.4; each char 
+                                                    is an achar */
+  dchar_t          volume_id[ISO_MAX_VOLUME_ID]; /**< Section 8.5.5; each char
+                                                    is a dchar */
   char             unused2[8];
   iso733_t         volume_space_size;            /**< total number of 
                                                     sectors */
@@ -398,40 +403,49 @@ struct iso9660_svd_s {
   iso723_t         volume_set_size;              /**< often 1 */
   iso723_t         volume_sequence_number;       /**< often 1 */
   iso723_t         logical_block_size;           /**< sector size, e.g. 2048 */
-  iso733_t         path_table_size;              /**< bytes in path table */
-  iso731_t         type_l_path_table;            /**< first sector of little-
-                                                      endian path table */
-  iso731_t         opt_type_l_path_table;        /**< first sector of optional
+  iso733_t         path_table_size;              /**< 8.5.7; bytes in path 
+                                                    table */
+  iso731_t         type_l_path_table;            /**< 8.5.8; first sector of 
                                                     little-endian path table */
-  iso732_t         type_m_path_table;            /**< first sector of big-
-                                                    endian path table */
-  iso732_t         opt_type_m_path_table;        /**< first sector of optional
+  iso731_t         opt_type_l_path_table;        /**< 8.5.9; first sector of 
+                                                    optional little-endian 
+                                                    path table */
+  iso732_t         type_m_path_table;            /**< 8.5.10; first sector of 
                                                     big-endian path table */
-  iso9660_dir_t    root_directory_record;        /**< See section 9.1 of
-                                                    ISO 9660 spec. */
-  dchar_t          volume_set_id[ISO_MAX_VOLUMESET_ID];    /**< dchars */
-  achar_t          publisher_id[ISO_MAX_PUBLISHER_ID];  /**< Publisher of
-                                                         volume. If the first
-                                                         character is '_' 0x5F,
-                                                         the remaining bytes
-                                                         specify a file 
-                                                         containing the user.
-                                                         If all bytes are " "
-                                                         (0x20) no publisher 
-                                                         is specified. See 
-                                                         section 8.4.20 of
-                                                         ECMA 119 */
-  achar_t          preparer_id[ISO_MAX_PREPARER_ID]; /**< preparer of
-                                                         volume. If the first
-                                                         character is '_' 0x5F,
-                                                         the remaining bytes
-                                                         specify a file 
-                                                         containing the user.
-                                                         If all bytes are " "
-                                                         (0x20) no preparer 
-                                                         is specified. 
-                                                         See section 8.4.21 
-                                                         of ECMA 119 */
+  iso732_t         opt_type_m_path_table;        /**< 8.5.11; first sector of 
+                                                    optional big-endian path 
+                                                    table */
+  iso9660_dir_t    root_directory_record;        /**< See section 8.5.12 and
+                                                    9.1 of ISO 9660 spec. */
+  char             root_directory_filename;      /**< Is '\\0' or root
+                                                  directory. Also pads previous
+                                                  field to 34 bytes */
+  dchar_t          volume_set_id[ISO_MAX_VOLUMESET_ID];    /**< 8.5.13; 
+                                                              dchars */
+  achar_t          publisher_id[ISO_MAX_PUBLISHER_ID]; /**< 
+                                                          Publisher of volume.
+                                                          If the first char-
+                                                          aracter is '_' 0x5F,
+                                                          the remaining bytes
+                                                          specify a file 
+                                                          containing the user.
+                                                          If all bytes are " "
+                                                          (0x20) no publisher 
+                                                          is specified. See 
+                                                          section 8.5.14 of
+                                                          ECMA 119 */
+  achar_t          preparer_id[ISO_MAX_PREPARER_ID]; /**< 
+                                                        Data preparer of
+                                                        volume. If the first
+                                                        character is '_' 0x5F,
+                                                        the remaining bytes
+                                                        specify a file 
+                                                        containing the user.
+                                                        If all bytes are " "
+                                                        (0x20) no preparer 
+                                                        is specified. 
+                                                        See section 8.5.15
+                                                        of ECMA 119 */
   achar_t          application_id[ISO_MAX_APPLICATION_ID]; /**< application
                                                          use to create the
                                                          volume. If the first
@@ -442,19 +456,19 @@ struct iso9660_svd_s {
                                                          If all bytes are " "
                                                          (0x20) no application 
                                                          is specified.
-                                                         See section of 8.4.22 
+                                                         See section of 8.5.16
                                                          of ECMA 119 */
   dchar_t          copyright_file_id[37];     /**< Name of file for
                                                  copyright info. If
                                                  all bytes are " "
                                                  (0x20), then no file
                                                  is identified.  See
-                                                 section 8.4.23 of ECMA 119
+                                                 section 8.5.17 of ECMA 119
                                                  9660 spec. */
-  dchar_t          abstract_file_id[37];      /**< See section 8.4.24 of 
+  dchar_t          abstract_file_id[37];      /**< See section 8.5.18 of 
+                                                 ECMA 119. */
+  dchar_t          bibliographic_file_id[37]; /**< See section 8.5.19 of 
                                                  ECMA 119. */
-  dchar_t          bibliographic_file_id[37]; /**< See section 7.5 of 
-                                                 ISO 9660 spec. */
   iso9660_ltime_t  creation_date;             /**< date and time of volume
                                                  creation. See section 8.4.26.1
                                                  of the ECMA 119 spec. */
@@ -471,7 +485,7 @@ struct iso9660_svd_s {
                                                  spec. */
   iso711_t         file_structure_version;    /**< value 1 for ECMA 119 */
   uint8_t           unused4[1];                /**< unused - value 0 */
-  char             application_data[512];     /**< Application can put 
+  char             application_data[512];     /**< 8.5.20 Application can put 
                                                  whatever it wants here. */
   uint8_t          unused5[653];              /**< Unused - value 0 */
 } GNUC_PACKED;
@@ -601,9 +615,22 @@ typedef struct _iso9660_s iso9660_t;
                                           uint16_t i_fuzz);
   
   /*!
-    Seek to a position and then read n bytes. Size read is returned.
+    Seek to a position and then read i_size blocks. 
+
+    @param p_iso the ISO-9660 file image to get data from
+
+    @param ptr place to put returned data. It should be able to store
+    a least i_size bytes
+
+    @param start location to start reading from
+
+    @param i_size number of blocks to read. Each block is ISO_BLOCKSIZE bytes
+    long.
+
+    @return number of bytes (not blocks) read
+
   */
-  long int iso9660_iso_seek_read (const iso9660_t *p_iso, void *ptr, 
+  long int iso9660_iso_seek_read (const iso9660_t *p_iso, /*out*/ void *ptr, 
                                   lsn_t start, long int i_size);
   
   /*!
@@ -820,6 +847,21 @@ iso9660_stat_t *iso9660_ifs_find_lsn(iso9660_t *p_iso, lsn_t i_lsn);
 
 /*!
   Return file status for psz_path. NULL is returned on error.
+
+  @param p_cdio the CD object to read from
+
+  @param psz_path filename path to look up and get information about
+
+  @return ISO 9660 file information
+
+  Important note:
+
+  You make get different results looking up "/" versus "/." and the
+  latter may give more complete information. "/" will take information
+  from the PVD only, whereas "/." will force a directory read of "/" and
+  find "." and in that Rock-Ridge information might be found which fills
+  in more stat information. Ideally iso9660_fs_stat should be fixed.
+  Patches anyone?
  */
 iso9660_stat_t *iso9660_fs_stat (CdIo_t *p_cdio, const char psz_path[]);
   
@@ -874,7 +916,7 @@ char * iso9660_get_application_id(iso9660_pvd_t *p_pvd);
   is some problem in getting this and false is returned.
 */
 bool iso9660_ifs_get_application_id(iso9660_t *p_iso,
-                                    /*out*/ char **p_psz_app_id);
+                                    /*out*/ cdio_utf8_t **p_psz_app_id);
 
 /*!  
   Return the Joliet level recognized for p_iso.
@@ -912,7 +954,7 @@ lsn_t iso9660_get_dir_extent(const iso9660_dir_t *p_idr);
     is some problem in getting this and false is returned.
   */
   bool iso9660_ifs_get_preparer_id(iso9660_t *p_iso,
-                                   /*out*/ char **p_psz_preparer_id);
+                                   /*out*/ cdio_utf8_t **p_psz_preparer_id);
   
   /*!
     Return a string containing the PVD's publisher id with trailing
@@ -925,7 +967,7 @@ lsn_t iso9660_get_dir_extent(const iso9660_dir_t *p_idr);
     is some problem in getting this and false is returned.
   */
   bool iso9660_ifs_get_publisher_id(iso9660_t *p_iso,
-                                    /*out*/ char **p_psz_publisher_id);
+                                    /*out*/ cdio_utf8_t **p_psz_publisher_id);
   
   uint8_t iso9660_get_pvd_type(const iso9660_pvd_t *p_pvd);
   
@@ -951,7 +993,7 @@ lsn_t iso9660_get_dir_extent(const iso9660_dir_t *p_idr);
     is some problem in getting this and false is returned.
   */
   bool iso9660_ifs_get_system_id(iso9660_t *p_iso,
-                                 /*out*/ char **p_psz_system_id);
+                                 /*out*/ cdio_utf8_t **p_psz_system_id);
   
   
   /*! Return the LSN of the root directory for pvd.
@@ -970,7 +1012,7 @@ lsn_t iso9660_get_dir_extent(const iso9660_dir_t *p_idr);
     is some problem in getting this and false is returned.
   */
   bool iso9660_ifs_get_volume_id(iso9660_t *p_iso,
-                                 /*out*/ char **p_psz_volume_id);
+                                 /*out*/ cdio_utf8_t **p_psz_volume_id);
   
   /*!
     Return the volumeset ID in the PVD.
@@ -983,7 +1025,7 @@ lsn_t iso9660_get_dir_extent(const iso9660_dir_t *p_idr);
     is some problem in getting this and false is returned.
   */
   bool iso9660_ifs_get_volumeset_id(iso9660_t *p_iso,
-                                    /*out*/ char **p_psz_volumeset_id);
+                                    /*out*/ cdio_utf8_t **p_psz_volumeset_id);
   
   /* pathtable */
   
