@@ -1,5 +1,5 @@
 #!/bin/sh
-#$Id: check_iso.sh.in,v 1.10 2005/09/15 06:36:01 rocky Exp $
+#$Id: check_iso.sh.in,v 1.11 2006/06/03 00:04:54 rocky Exp $
 
 if test -z $srcdir ; then
   srcdir=`pwd`
@@ -20,7 +20,7 @@ RC=$?
 check_result $RC 'iso-info basic test' "$ISO_INFO $opts"
 
 opts="--image ${srcdir}/${fname}.iso --extract $fname "
-test_iso_read  "$opts" ${fname} ${srcdir}/../COPYING
+test_iso_read  "$opts" ${fname} ${srcdir}/copying.gpl
 RC=$?
 check_result $RC 'iso-read basic test' "$ISO_READ $opts"
 
